#!usr/bin/perl -w
########################################################################
#                                                                      #
# Author:       John                                                   #
# Contact:      support@hrcompletemanager.com                          #
# Purpose:      Creates a a batch file that updates the registry       # 
#               with current date and time values.                     #
#                                                                      #
# Copyright:    hrcompletemanager.com 2022                            #
#               This is free software. Do with is what you wish. It    #
#               comes with no guarantees or warranties. The  author is #
#               not responsible for anything. If you use it and find   #
#               it useful then a thank you would be nice. If you       #
#               use it to make a lot of money then congratulations.    #
#                                                                      #
# WARNING:      WE USE THIS PROGRAM A LOT                              #
#               IT DOES MORE OR LESS EXACTLY WHAT WE WANT              #
#               IT MAY NOT BE SUITABLE FOR YOU                         #
#               USE IT AT YOUR OWN RISK                                #
#                                                                      #
########################################################################
=head1 DESCRIPTION

This utility takes the names of files to backup from pairs of list files:
     xxx.lst for non-recursive stuff
     xxx_r.lst for recursive stuff

     (where xxx is passed to the function)
     
I prefer to keep these list files in the same directory as this perl script 
but this is user configurable. The list files are edited to contain names 
of files that you want to backup. You can have as many pairs of these list 
files as you wish. If either (or both) of the pair are missing the script 
will log this and just carry on.

I have created a number of these pairs for use with different types of my files 
(eg mydocuments.lst, photos.lst, systemfiles.lst etc).

The point of having config files in pairs like this is that the script creates
one archive file for each pair and it will inherit the same name. It seemed 
simplest to do it this way and I like simple.
=cut

use Win32;
use strict;
use Env;

#USAGE
my $usage = "\nERROR: Usage: $0 mode filename \neg backitup batch espresso\n\n";
@ARGV eq 1 or die "$usage";
    
#=======================================================================
# USER CONFIGURABLE SECTION
# Because the Env package is used you might prefer to use environment 
# variables instead of this section. We have environment variables set 
# for the paths to the backitup root and logfile so the next 2 variables
# are commented out 
#=======================================================================
# Use the next line to set the backup root path
    #my $BACKITUP_LOG = "c:/data/backitup.log"; 
# Use the next line to set the backup log path
    #my $BACKITUP_ROOT = "c:/data";
# Use the next line if you want to prefix file name with userid
    my $USERID = Win32::LoginName;
# Use the next line if you want to prefix file with nodename
    my $NODEID = Win32::NodeName;
# Path seperator
    my $PS = "/";
# Where the list files are kept
    my $LSTFILE_PATH = ".";
# For non-recursive stuff use a list file with a name like this
    my $LSTFILE = "$LSTFILE_PATH"."$PS"."$ARGV[0]".".lst";    
# For recursive stuff use a list file with a name like this
    my $LSTFILE_R = "$LSTFILE_PATH"."$PS"."$ARGV[0]"."_r.lst";  
# Use the following command to define the zip exe for recursive stuff
    my $ZIPPER_R = "\"c:/program files (x86)/winzip/wzzip\" -p -r -whs -o";    
# Use the following command to define the zip exe for non-recursive stuff
    my $ZIPPER = "\"c:/program files (x86)/winzip/wzzip\" -P -whs -o";    
# suffix for the archive file
    my $ZIPTYPE = ".zip";        
# The following variable describes the type of naming required for the files and directories  
    # "FULL" for full dirs and subdirs
    # "MONTHS" for dirs like yyyy_mm only
    # Any other value will result in a flat structure
    my $MAKE_DIR_TREE = "FULL";                  
# END OF USER CONFIGURABLE SECTION
#=======================================================================

# DEFINE THE MONTH NAMES
    my %hMonthName = (
        '01' => 'Jan',  
        '02' => 'Feb',
        '03' => 'Mar',
        '04' => 'Apr',
        '05' => 'May',
        '06' => 'Jun',
        '07' => 'Jul',
        '08' => 'Aug',
        '09' => 'Sep',
        '10' => 'Oct',
        '11' => 'Nov',
        '12' => 'Dec');

# DEFINE THE DAY NAMES
    my %hDayName = (
        '01' => "Sun",
        '02' => "Mon",
        '03' => "Tue",
        '04' => "Wed",
        '05' => "Thu",
        '06' => "Fri",
        '07' => "Sat",);

# SET UP THE DATE AND TIME VALUES
    (my $sec,my $min,my $hour,my $mday,my $mon,my $year,my $wday,my $yday)=localtime(time);

# CHANGE DATE VALUES TO DESIRED FORMAT
    $sec = sprintf("%.2d",$sec);
    $min = sprintf("%.2d",$min);
    $hour = sprintf("%.2d",$hour);
    $wday = sprintf("%.2d",($wday+1));
    $mday = sprintf("%.2d",$mday);
    $mon = sprintf("%.2d",($mon+1));
    $year += 1900;
    my $day = $hDayName{ $wday };
    my $month = $hMonthName{ $mon };
    
    $yday = sprintf("%.3d",$yday);
    print "DAY OF YEAR: $yday";

# CHECK THERE IS SOMETHING TO BACKUP FIRST
    print "\nINFO: Checking there is something to backup.\n";
    my $errorCounter = 0;
    # FIRST CHECK THE EXISTENCE OF A FILE FOR NON-RECURSIVE STUFF
    if (-e $LSTFILE) {
    }
    else {
        print "\nWARN: The following expected file does not exist: \n $LSTFILE\n";
        $errorCounter = $errorCounter + 1;
    }
    # NOW CHECK THE EXISTENCE OF A FILE FOR RECURSIVE STUFF
    if (-e $LSTFILE_R) {
    }
    else {
        print "\nWARN: The following expected file does not exist: \n $LSTFILE_R\n";
        $errorCounter = $errorCounter + 1;
    }
    if ($errorCounter == 2){
        # IF YOU GOT HERE THERE ARE NO LIST FILES AND, THEREFORE, NOTHING TO DO
        die $!;
    }    

# DO THE DIRECTORY STRUCTURE CREATION
 my $path = $BACKITUP_ROOT.$PS.$year."_".$mon;
 my $subpath = $path.$PS.$mday."_".$month."_".$day;
 my $filename = "fred";

    if($MAKE_DIR_TREE eq "FULL"){
        print "\nINFO: Using a FULL Directory structure for the backups.\n";
        &CreatePath ($path);                                        
        &CreatePath ($subpath);
        $filename = $subpath.$PS.$year."_".$mon."_".$mday."_".$day."_".$USERID."_".$NODEID."_".$hour.".".$min."_".$ARGV[0].$ZIPTYPE;
    }elsif($MAKE_DIR_TREE eq "MONTHS"){
        print "\nINFO: Using a Monthly Directory structure for the backups.\n";
        $filename = $path.$PS.$year."_".$mon."_".$mday."_".$day."_".$USERID."_".$NODEID."_".$hour.".".$min."_".$ARGV[0].$ZIPTYPE;
        &CreatePath ($path);                                        
    }else{
        print "\nINFO: Using a Flat structure for the backups.\n";
        $filename = $BACKITUP_ROOT.$PS.$year."_".$mon."_".$mday."_".$day."_".$USERID."_".$NODEID."_".$hour.".".$min."_".$ARGV[0].$ZIPTYPE;
    }
# DO THE BACKUP
    &LogBackup;
    &PerformBackup;
    exit 0;
    
#=================================================
# START OF SUBS
#-------------------------------------------------
sub LogBackup{
    open (FILEHANDLE, ">>$BACKITUP_LOG") or die ("Cannot open $BACKITUP_LOG");
    print FILEHANDLE "Backup started => $filename\n";
    close (FILEHANDLE);
}
#-------------------------------------------------
sub PerformBackup{
    print "\nINFO: Backup started.\n";
    
    # FIRST THE NON-RECURSIVE STUFF
    if (-e $LSTFILE) {
        my @args = ("$ZIPPER $filename \@$LSTFILE");
        system(@args) == 0 or die "system @args failed: $?";
    }
    else {
    }

    # NOW THE RECURSIVE STUFF
    if (-e $LSTFILE_R) {
        my @args = ("$ZIPPER_R $filename \@$LSTFILE_R");
        system(@args) == 0 or die "system @args failed: $?";
    }
    else {
    }
}
#-------------------------------------------------
sub CreatePath {
if (-e $_[0]) {
    print "\nINFO: The following directory will be used\n    $_[0]\n";
}
else {
mkdir  ($_[0],0777), print "\nINFO: The following directory was created:\n    $_[0]\n" || die "\nERROR: Cannot make directory:\n    $_[0] $!\n";
}        
}
#-------------------------------------------------
# END OF SUBS
#=================================================